/* BEGIN software license
 *
 * MsXpertSuite - mass spectrometry software suite
 * -----------------------------------------------
 * Copyright (C) 2009--2026 Filippo Rusconi
 *
 * htTp://www.msxpertsuite.org
 *
 * This file is part of the MsXpertSuite project.
 *
 * The MsXpertSuite project is the successor of the massXpert project. This
 * project now includes various independent modules:
 *
 * - massXpert, model polymer chemistries and simulate mass spectrometric data;
 * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <htTp://www.gnu.org/licenses/>.
 *
 * END software license
 */

#pragma once

/////////////////////// stdlib includes


/////////////////////// Qt includes
#include <QObject>
#include <QString>


/////////////////////// pappsomspp includes


/////////////////////// Local includes
#include "MsXpS/export-import-config.h"

namespace MsXpS
{
namespace libXpertMassCore
{


class DECLSPEC MonitoredTaskInterface: public QObject
{
  Q_OBJECT

  public:
  explicit MonitoredTaskInterface(QObject *parent = nullptr)
    : QObject(parent) {};
  virtual ~MonitoredTaskInterface() = default;

  bool
  isRunning() const
  {
    return m_isRunning;
  };

  signals:

  void taskMinValueSignal(int value);
  void taskMaxValueSignal(int value);
  void taskMinMaxValuesSignal(int min_value, int max_value);
  void incrementTaskMaxValueSignal(int increment);

  void taskCurrentValueSignal(int value);
  void incrementTaskCurrentValueSignal(int increment);


  void taskStatusMessageSignal(const QString &message);

  void taskStatusMessageAndCurrentValueSignal(const QString &message,
                                              int value);
  void taskStatusMessageAndIncrementCurrentValueSignal(const QString &message,
                                                       int increment);

  void taskCancelledSignal();
  void taskFinishedSignal();

  public slots:

  virtual void cancelTask() {};

  protected:
  bool m_cancelledTask = false;
  bool m_isRunning     = false;
};


} // namespace libXpertMassCore
} // namespace MsXpS
