/* BEGIN software license
 *
 * MsXpertSuite - mass spectrometry software suite
 * -----------------------------------------------
 * Copyright (C) 2009--2026 Filippo Rusconi
 *
 * http://www.msxpertsuite.org
 *
 * This file is part of the MsXpertSuite project.
 *
 * The MsXpertSuite project is the successor of the massXpert project. This
 * project now includes various independent modules:
 *
 * - massXpert, model polymer chemistries and simulate mass spectrometric data;
 * - mineXpert, a powerful TIC chromatogram/mass spectrum viewer/miner;
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * END software license
 */

#pragma once

/////////////////////// stdlib includes


/////////////////////// Qt includes
#include <QObject>
#include <QList>
#include <QMap>
#include <QFileInfo>


/////////////////////// pappsomspp includes
#include "pappsomspp/core/trace/trace.h"


/////////////////////// Local includes
#include "MsXpS/export-import-config.h"
#include "MsXpS/libXpertMassCore/AbstractLowMassDeconvolver.hpp"

namespace MsXpS
{
namespace libXpertMassCore
{


class DECLSPEC LowMassDeconvolverWorker: public QObject
{
  Q_OBJECT

  public:
  struct Data
  {
    pappso::Trace centroids;
    libXpertMassCore::LowMassDeconvolverParameters deconvolverParams;
    void *centroidsSourcePlottable;
    bool fullHeavyIsotopeLabelling;
    QString sampleName;
    QString sampleColorString;
    bool shouldSynthesizeMassSpectrum;
    bool shouldIdentifyAnalytes;
  };

  explicit LowMassDeconvolverWorker(QObject *parent = nullptr);

  void setData(const Data &data);

  public slots:
  void process();

  signals:
  void finishedSignal(QList<libXpertMassCore::DeconvolutedFeature> results,
                      void *centroids_source_plottable,
                      const QString &sample_name,
                      const QString &label_color_string,
                      bool should_synthesize,
                      bool should_identify);
  void progressSignal(int percent);
  void errorSignal(const QString &message);
  void statusMessageSignal(const QString &message);

  private:
  Data m_data;
};


} // namespace libXpertMassCore
} // namespace MsXpS
